#!/bin/bash
#
# Copyright (c) 2014, the rcondo author(s)
#
# Permission to use, copy, modify, and/or distribute this software for any
#    purpose with or without fee is hereby granted, provided that the above
#    copyright notice and this permission notice appear in all copies.

set -e

REXEC_TMPFILE=''
REXEC_INTERPRETER='bash -s'
IS_VERBOSE=0
ALREADY_INCLUDED=''

fatal() {
	echo "FATAL: $1" >&2
	exit 1
}

verbose() {
	if [[ $IS_VERBOSE -eq 1 ]] ; then
		echo "LOG: $1" >&2
	fi
}

rparse() {
	
	while IFS= read -r line ; do
		
		#include {glob}
		
		local regex="^#include (.*)"
		if [[ "$line" =~ $regex ]] ; then
		
			for includefile in ${BASH_REMATCH[1]} ; do
				
				if echo "$ALREADY_INCLUDED" | grep -q "$includefile" ; then
					verbose "Skipping already-included file '${includefile}'..."
					continue
				
				elif [ -f "$includefile" ] ; then
					verbose "Parsing nested file '${includefile}'..."
					ALREADY_INCLUDED="$(
						echo "$ALREADY_INCLUDED"
						echo "${includefile}"
					)"
					rparse < "$includefile"
					continue
					
				else
					fatal "Local file '$includefile' doesn't exist"
				fi
			done
			
			continue
		fi
		
		#packfile {localpath} {remotepath}
		
		local regex="^#packfile (.*) (.*)"
		if [[ "$line" =~ $regex ]] ; then
			local localfile="${BASH_REMATCH[1]}"
			local remotefile="${BASH_REMATCH[2]}"
			
			if [ ! -f "$localfile" ] ; then
				fatal "Local file $localfile doesn't exist"
			fi
			
			echo "# Unpacking file"
			echo "base64 -di <<__EOD | gzip -d > $remotefile"
			cat "$localfile" | gzip | base64
			echo "__EOD"
						
			continue
		fi
		
		#packdir {localdir} {remotedir}
		
		local regex="^#packdir (.*) (.*)"
		if [[ "$line" =~ $regex ]] ; then
			local localdir="${BASH_REMATCH[1]}"
			local remotedir="${BASH_REMATCH[2]}"
			
			if [[ ! -d "$localdir" ]] ; then
				fatal "Local directory $localdir doesn't exist"
			fi
			
			echo "# Unpacking directory"
			echo "pushd $remotedir >/dev/null"
			echo "base64 -di <<__EOD | tar zxp"
			pushd "$localdir" >/dev/null
			tar cz --owner=0 --group=0 --mode='a-x,a=rwX' . | base64
			popd >/dev/null
			echo "__EOD"
			echo "popd >/dev/null"
						
			continue
		fi
		
		# Literal
		
		echo "$line"
		
	done
}

rexec_cleanup() {
	if [[ -f "${REXEC_TMPFILE}" ]] ; then
		rm "${REXEC_TMPFILE}"
	fi
}

rexec() {
	
	rexec_cleanup
	trap rexec_cleanup EXIT
	
	REXEC_TMPFILE=$(mktemp)	
	cat > "${REXEC_TMPFILE}"
	
	local sshopts=$(sed -n 's/#ssh-options \(.*\)/\1/p' < "${REXEC_TMPFILE}")
	
	ssh "$sshopts" "${REXEC_INTERPRETER}" < "${REXEC_TMPFILE}"
	
}

usage() {
	cat <<EOU
rcondo.sh [options] file1 [file2 ...]
Options:
   -e                  Execute script remotely
   --rcmd={command}    Set remote interpreter (default 'bash -s')
   -v                  Verbose (also sets --rcmd='bash -x -s')
EOU
	exit 1
}

main() {
	
	if [[ $# -eq 0 ]] ; then
		usage
	fi
	
	local DOEXECUTE=0
	local rcmdregex="--rcmd=(.*)"
	
	for arg in "$@" ; do
		
		if [[ "$arg" == "-e" ]] ; then
			DOEXECUTE=1
		
		elif [[ "$arg" =~ $rcmdregex ]] ; then
			REXEC_INTERPRETER="${BASH_REMATCH[1]}"
		
		elif [[ "$arg" == "-v" ]] ; then
			REXEC_INTERPRETER='bash -x -s'
			IS_VERBOSE=1
		
		elif [[ -f "$arg" ]] ; then
		
			verbose "Parsing file '$arg'..."
		
			# File argument
			if [[ "$DOEXECUTE" -eq 0 ]] ; then
				# Build-only
				rparse < "$arg"
			else
				# Build-and-execute
				# (bash: redirections applied before pipeline)
				rparse < "$arg" | rexec
			fi
		
		else
			fatal "Unexpected argument or not a file \"${arg}\"" 
			
		fi
	
	done
	
}

main "$@"
